# coding: utf-8

from django.db import models
from django.db.models.signals import pre_delete
from django.dispatch import receiver
from django.conf import settings
from m3.db import BaseEnumerate

from kinder.core.models import LogableModel, SimpleDictionary
from kinder.core.audit_log.managers import AuditLog

from kinder.core.privilege.models import Privilege
from kinder.core.declaration.models import (
    Declaration,
    DeclarationPrivilege)
from kinder.core.children.models import Delegate
from kinder.users.models import UserProfile

from lipetsk_specifics.constants import DepartmentAttribute
from lipetsk_specifics.utils import get_upload_dir


class PrivilegeOwnerEnum(BaseEnumerate):
    PARENT = 1
    CHILDREN = 2
    DELEGATE = 3

    values = {
        PARENT: u'Родитель',
        CHILDREN: u"Ребенок",
        DELEGATE: u"Представитель"
    }

    NEED_ATTRIBUTES = (
        PARENT,
        DELEGATE
    )

    NEED_DELEGATE = (
        PARENT,
        DELEGATE
    )


class Department(SimpleDictionary):
    """
    Справочник "Ведомства"
    """
    #TODO:На самом деле, к одной льготе принадлежит только 1 ведомство,
    # нужен ключ!!!
    privileges = models.ManyToManyField(Privilege)

    class Meta:
        verbose_name = u"Ведомства"


class DepartmentAttributes(LogableModel):
    """
    Атрибуты ведомства
    """

    department = models.ForeignKey(Department, null=False)

    attribute = models.CharField(
        max_length=256,
        choices=DepartmentAttribute.get_choices(),
        verbose_name=u"Атрибут")

    audit_log = AuditLog()

    class Meta:
        verbose_name = u"Атрибуты ведомства"
        unique_together = ('department',
                           'attribute')


class PrivilegeConfirmationAttributes(LogableModel):
    """
    Дополнительные атрибуты для подтверждения льготы
    """

    delegate = models.ForeignKey(
        Delegate,
        null=True, verbose_name=u'Обладатель льготы')
    declaration_privilege = models.ForeignKey(
        DeclarationPrivilege,
        null=False)

    confirmed = models.BooleanField(
        default=False)

    privilege_owner = models.SmallIntegerField(
        u"Принадлежность льготы",
        choices=PrivilegeOwnerEnum.get_choices(),
        null=True)

    # Общие поля дополняющие поля из DeclarationPrivilege
    document_issued_by = models.CharField(
        max_length=256,
        verbose_name=u"Кем выдан",
        null=True, blank=True)
    document_date = models.DateField(
        verbose_name=u"Дата выдачи",
        null=True, blank=True)

    # Поля связанные с настраиваемыми аттрибутами ведомства
    birthplace_country = models.CharField(
        max_length=256,
        verbose_name=u"Страна",
        null=True, blank=True)
    birthplace_region = models.CharField(
        max_length=256,
        verbose_name=u"Регион",
        null=True, blank=True)
    birthplace_city = models.CharField(
        max_length=256,
        verbose_name=u"Населенный пункт",
        null=True, blank=True)
    personal_number = models.CharField(
        max_length=256,
        verbose_name=u"Личный номер",
        null=True, blank=True)
    force_kind = models.CharField(
        max_length=256,
        verbose_name=u"Принадлежность к виду или роду войск",
        null=True, blank=True)
    military_unit = models.CharField(
        max_length=256,
        verbose_name=u"Воинская часть (Подразделение)",
        null=True, blank=True)
    dismissal_date = models.DateField(
        verbose_name=u"Дата увольнения",
        null=True, blank=True)
    rank = models.CharField(
        max_length=256,
        verbose_name=u"Звание",
        null=True, blank=True)
    portal = models.BooleanField(
        verbose_name=u'Признак что льгота пришла с портала',
        default=False)

    audit_log = AuditLog()

    def __unicode__(self):
        return u"{0} ({1})".format(
            self.declaration_privilege.privilege.name,
            self.delegate.fullname if self.delegate else '')

    class Meta:
        verbose_name = u"Атрибуты подтверждение льготы"
        unique_together = ('delegate',
                           'declaration_privilege')


class RejectReason(SimpleDictionary):
    """
    Справочник "Причины отказа"
    """

    class Meta:
        verbose_name = u"Причины отказа"


class DeclarationRejectReason(LogableModel):
    """
    Связка между заявлениями и причинами отказа
    """

    declaration = models.ForeignKey(
        Declaration,
        verbose_name=u"Заявление")
    reject_reason = models.ForeignKey(
        RejectReason,
        verbose_name=u"Причина отказа",
        null=True, blank=True)

    audit_log = AuditLog()

    class Meta:
        verbose_name = u"Причины отказа"
        unique_together = ('declaration',
                           'reject_reason')


class PassingSmev(LogableModel):
    u"""Обращения в не электронные ведомства."""

    department = models.ForeignKey(Department, verbose_name=u'Ведомство')
    profile = models.ForeignKey(UserProfile, verbose_name=u'Пользователь')
    request = models.FileField(
        u'Запрос', upload_to=get_upload_dir, max_length=150)
    time = models.DateTimeField(
        u'Дата и время', auto_now_add=True, db_index=True)
    result = models.FileField(
        u'Результат', upload_to=get_upload_dir, max_length=150, null=True,
        blank=True)

    class Meta:
        verbose_name = u'Обращения в не электронные ведомства'


class DeclarationPassingSmev(LogableModel):
    u"""Привязка обращений в не электронные ведомства и заявлений."""

    passing_smev = models.ForeignKey(
        PassingSmev, verbose_name=u'Обращение в не электронные ведомства')
    declaration = models.ForeignKey(Declaration, verbose_name=u'Заявление')


class ReportType(BaseEnumerate):
    u"""Перечисление типов отчета.

    Используетсся в модели доп. атрибутов льготы.
    """

    FIRST = 1
    SECOND = 2
    THIRD = 3
    FOURTH = 4

    values = {
        FIRST: u'ФИО, дата рождения, адрес, СНИЛС',
        SECOND: u'ФИО, дата рождения, место рождения, адрес, СНИЛС, '
                u'наименование подразделения, '
                u'в котором служит (служил) обладатель льготы, звание',
        THIRD: u'ФИО, дата рождения, место рождения, адрес, СНИЛС, '
               u'ОВД, выдавшее удостоверение, звание',
        FOURTH: u'ФИО, дата рождения, личный номер, '
                u'принадлежность к виду или роду войск, воинская часть, '
                u'дата увольнения, документ, '
                u'удоствоверяющий личность, удостоверение, военный билет'
    }


class ExtendedPrivilege(LogableModel):
    u"""Модель дополнительных атрибутов льготы."""

    privilege = models.OneToOneField('privilege.Privilege')
    name_to_query_at_depart = models.CharField(
        u'Наименования для запроса в ведомства',
        max_length=500, blank=True)

    report_type = models.IntegerField(u'Тип отчета',
                                      default=ReportType.FIRST,
                                      choices=ReportType.get_choices())


@receiver(pre_delete, sender=DeclarationPrivilege)
def pre_delete_privilege_lipetsk(sender, instance, **kwargs):
    """Перед удалением "Льготы в заявке" необходимо удалить
    все связные записи на подтверждение. Удаляем так,
    чтобы отразилось в МИС-логе"""
    for pca in instance.privilegeconfirmationattributes_set.all():
        pca.delete()
